const {
  ImportMethods,
  MissingRequiredParamError,
  Type
} = require( 'o-toolbox' )
const BluePrintCommon = require( './BluePrintCommon' )
const ProgramBluePrint = require( './ProgramBluePrint' )

class BluePrint {
  constructor (params = {}) {
    const { parent } = params
    MissingRequiredParamError.raiseIfMissing({ param: parent, paramName: 'parent' })
    this.parent = Type.respondsTo( parent, 'asReference' )
                    ? parent.asReference()
                    : parent
    this.publicCommands = [
      'evaluate',
      'getCurrentPath',
      'getRouter',
      'getUID',
      'gotoPath',
      'gotoPathLocally',
      'invalidate',
      'route',
      'routeWithMethod',
      'respondsTo',
      'setPath'
    ]
    this.uid = ProgramBluePrint.createUID()
    this.initialize( params )

    return this.asReference()
  }

  acceptsCommand ({ command }) {
    return Type.respondsTo( this, command ) &&
           this.publicCommands.includes( command )
  }

  evaluate ({ command, params }) {
    const acceptsCommand = this.acceptsCommand({ command })
    if ( acceptsCommand ) {
      return Reflect.apply(
          this[ command ],
          this,
          params
        )
    }
    if ( this.parent ) { return this.parent.evaluate({ command, params }) }
    this.raiseCommandNotFoundError({ command, params })
  }

  getClassName () {
    return this.constructor.name
  }

  getCurrentPath () {
    return this.parent.getCurrentPath()
  }

  getUID () {
    return this.uid
  }

  gotoPath ({ path }) {
    this.parent.gotoPath({ path })
  }

  invalidate () {
    return this.parent.invalidate()
  }

  gotoPathLocally ({ path }) {
    return this.parent.gotoPathLocally({ path })
  }

  setPath ({ path }) {
    return this.parent.setPath({ path })
  }

  respondsTo ({ command }) {
    return this.acceptsCommand({ command }) ||
           (
             this.parent &&
             this.parent.respondsTo({ command })
           )
  }

  route () {
    return this.parent.getRouter()
             .route({ bluePrint: this })
  }

  routeWithMethod ({ methodName }) {
    return this.parent.getRouter()
             .routeWithMethod({ methodName, bluePrint: this })
  }
}
ImportMethods.all({ from: BluePrintCommon, into: BluePrint })

module.exports = BluePrint