const { Type, AbsentMethodError } = require( 'o-toolbox' )

class Router {
  route ({ bluePrint }) {
    const methodName = this.getRenderMethodName({ bluePrint })

    return this.routeWithMethod({ methodName, bluePrint })
  }

  getRenderMethodName ({ bluePrint }) {
    if ( !bluePrint.getRenderMethodName ) {
      this.raiseMissingGetRenderMethodNameError({ bluePrint })
    }
    const methodName = bluePrint.getRenderMethodName()
    if ( !methodName ) {
      this.raiseInvalidRenderMethodNameError({ bluePrint })
    }

    return methodName
  }

  routeWithMethod ({ methodName, bluePrint }) {
    return Type.respondsTo( this, methodName )
             ? Reflect.apply(
                 this[ methodName ],
                 this,
                 [ bluePrint.asReference() ]
               )
             : this.raiseMissingRenderMethodError({ router: this, methodName })
  }

  raiseMissingRenderMethodError ({ router, methodName }) {
    const message = `The class ${ router.constructor.name } must implement the method '${ methodName }(bluePrint)'`
    AbsentMethodError.raise({ message })
  }

  raiseMissingGetRenderMethodNameError ({ bluePrint }) {
    const message = `The class ${ bluePrint.constructor.name } must implement the method 'getRenderMethodName()'`
    AbsentMethodError.raise({ message })
  }

  raiseInvalidRenderMethodNameError ({ bluePrint }) {
    const message = `The class ${ bluePrint.constructor.name }.getRenderMethodName() method returned an empty string. Maybe it's missing the return statement?`
    AbsentMethodError.raise({ message })
  }
}

module.exports = Router